#!/bin/bash -e
#
#
# Daniel Huhardeaux devel@tootai.net
#
# v20220917 - version initiale
# v20220921 - le jeton n'est relu que si la durée de vie est atteinte
# v20220922 - rajout de la possibilité de logguer les informations
#           - si la date de création n'existe pas, la date d'accès (atime) est utilisée
#           - $tmpday4 ou ${jour4[0]} = "null." on ne fait rien
# v20220927 - simplification en passant les paramètres aux fonctions lorsque possible
# v20220928 - le jeton auth est à mettre dans un include (donnée privée) nommé rteauth.include
#             présent dans le même répertoire que ce script

# Documentation
#
# https://linuxconfig.org/how-to-parse-a-json-file-from-linux-command-line-using-jq
# https://www.monecowatt.fr/inscription-alerte-vigilance-coupure/
# https://data.rte-france.com/catalog/-/api/consumption/Ecowatt/v3.0 

# Execution via cron tous les 1/4h avec un décalage de 2 minutes SANS envoi de courriel,
# le script s'en charge en cas d'alerte-s en utilisant la fonction mail du paquet bsd-mailx
#
# */15 * * * * sleep 120 && /path/vers/le/script > /dev/null


myaccess=https://digital.iservices.rte-france.com
mysandbox=sandbox                                               # valeur à commenter pour le réel. Ex. mysandbox= #sandbox  
myapplication=open_api/ecowatt/v4/$mysandbox/signals
myinclude=rteauth.include
mypath=$(dirname $(realpath $0))

# Sorties
#
mytoken=$HOME/tmp/rtetoken
myoutput=$HOME/tmp/rteoutput
mytomail=$HOME/tmp/rtetomail
mylogfile=$HOME/tmp/rteecowatt.log

# Qui réceptionne les alertes courriel
#
mailalert="moi@exemple.com"

# Divers
#
verbose=1                                                        # 1 verbeux 2 plus verbeux
logfile=1
outputsize=0
display_all=0
status=("" "Verte" "Orange" "Rouge")
BAKIFS=$IFS

# Lecture du jeton délivré par RTE si le fichier existe et de taille supérieure à zéro
#

if [ -s "$mypath/$myinclude" ]; then
  # les lignes débutant par # ou vides ne sont pas prises en compte
  # si le jeton a été saisi comme une variable (var="jeton") on conserve la valeur et on supprime les "
  #
  myoauth=$(cat "$mypath/$myinclude"| egrep -v '^\s*(#|$)'|cut -d = -f2|tr -d \")
  [ $verbose -gt 1    ] && echo "Passage 1: $myoauth"

  # si le jeton a été saisi comme ("jeton") on supprime les "
  # si la ligne ne contient que la valeur du jeton (jeton) elle sera acceptée 
  #
  [ "x$myoauth" = "x" ] && myoauth=$(cat "$mypath/$myinclude"| egrep -v '^\s*(#|$)'|tr -d \")
  [ $verbose -gt 1    ] && echo "Passage 2: $myoauth"
  [ "x$myoauth" = "x" ] && echo -e "\nLe token n'est pas valide ou non défini, ABANDON\n" && exit 0
else
  echo -e "\nLe fichier $myinclude n'existe pas dans le répertoire de localisation de ce script ($mypath)"
fi


# Fonctions
#
get_token() {
  timenow=$(date +%s)                                            # date à l'instant en secondes depuis EPOCH

  # Si le fichier token existe, est un fichier régulier et sa taille est > à zéro
  #
  if [ -s $mytoken ]; then
    datebirth=$(stat -c %W $mytoken)                             # date de création en secondes depuis EPOCH
    [ $datebirth -eq 0 ] && datebirth=$(stat -c %X $mytoken)     # date d'accès     en secondes depuis EPOCH si la date de création est inconnue
    myexpires=$(cat $mytoken|jq .expires_in)                     # durée de validité en secondes

    if [ $verbose -gt 0 ]; then
      echo -e "\n*  Valeurs en secondes  *"
      echo "L'heure actuelle est ...: $timenow ($(echo `date -d @$timenow`))"
      echo "L'heure de création est : $datebirth ($(echo `date -d @$datebirth`))"
      echo "La durée de vie est de .: $myexpires (soit $((myexpires/60/60))h)"
      echo "Le jeton expire le .....: $(echo `date -d @$((datebirth+myexpires))`)"
    fi

    if [ $logfile -eq 1 ]; then
      echo -e "\n*  Valeurs en secondes  *" >> $mylogfile
      echo "L'heure actuelle est ...: $timenow ($(echo `date -d @$timenow`))" >> $mylogfile
      echo "L'heure de création est : $datebirth ($(echo `date -d @$datebirth`))" >> $mylogfile
      echo "La durée de vie est de .: $myexpires (soit $((myexpires/60/60))h)" >> $mylogfile
      echo "Le jeton expire le .....: $(echo `date -d @$((datebirth+myexpires))`)" >> $mylogfile
    fi

    # si le token est valide, on quitte la fonction après avoir récupérer le jeton
    #
    if [ $((datebirth+myexpires)) -gt $timenow ]; then
      myauth=$(cat $mytoken|jq .access_token|tr -d \")

      if [ $verbose -gt 0 ]; then
        echo -e "\n* Jeton en cours valide *"
        echo "Le token a comme valeur : $myauth"
      fi

      if [ $logfile -eq 1 ]; then
        echo -e "\n* Jeton en cours valide *" >> $mylogfile
        echo "Le token a comme valeur : $myauth" >> $mylogfile
      fi

      return 0
    fi
  fi

  # Suppression du fichier avant sa création afin de connaitre l'heure de celle ci
  #
  rm -f $mytoken
  echo "" > $mytoken
  curl -4 -o $mytoken -X POST -H "Content-Type: application/x-www-form-urlencode" -H "Authorization: Basic $myoauth" $myaccess/token/oauth/ 2>/dev/null
  echo "" >> $mytoken                                       # saut de ligne final

  myauth=$(cat $mytoken|jq .access_token|tr -d \")
  myexpires=$(cat $mytoken|jq .expires_in)                  # durée de validité en secondes
  myexpireshours=$((myexpires/60/60))                       # durée de validité en heures

  if [ $verbose -gt 0 ]; then
    echo "*     Nouveau jeton     *"
    echo "Le token a comme valeur : $myauth"
    echo "La durée de vie est de .: $myexpires (soit $((myexpires/60/60))h)"
    echo "Le jeton expire le .....: $(echo `date -d @$((timenow+myexpires))`)"
  fi

  if [ $logfile -eq 1 ]; then
    echo "*     Nouveau jeton     *" >> $mylogfile
    echo "Le token a comme valeur : $myauth" >> $mylogfile
    echo "La durée de vie est de .: $myexpires (soit $((myexpires/60/60))h)" >> $mylogfile
    echo "Le jeton expire le .....: $(echo `date -d @$((timenow+myexpires))`)" >> $mylogfile
  fi
}

get_signals() {
  rm -f $myoutput.tmp
  curl -4 -o $myoutput.tmp --oauth2-bearer $myauth $myaccess/$myapplication 2>/dev/null

  outputsize=$(wc -c $myoutput.tmp|cut -d " " -f1)
  if [ $verbose -gt 0 ]; then echo "Ont été réceptionnés ...: $outputsize octets"; fi

  if [ $outputsize -gt 1 ]; then
    echo "" >> $myoutput.tmp                    # saut de ligne final
    rm $myoutput 2>/dev/null
    mv $myoutput.tmp $myoutput
    if [ $verbose -gt 0 ]; then echo -e "\n***   Nouvelles données disponibles   ***"; fi
    if [ $logfile -eq 1 ]; then echo -e "\n***   Nouvelles données disponibles   ***" >> $mylogfile; fi
  else
    if [ $verbose -gt 0 ]; then echo -e "\nNote: les données précédentes ont été conservées"; fi
    if [ $logfile -eq 1 ]; then echo -e "\nNote: les données précédentes ont été conservées" >> $mylogfile; fi
    rm $myoutput.tmp
  fi
}

set_valuesPerDay() {
  j=1
  for i in $tmpval; do
    if [ $j -eq 1 ]; then jour1+=($i); fi
    if [ $j -eq 2 ]; then jour2+=($i); fi
    if [ $j -eq 3 ]; then jour3+=($i); fi
    if [ $j -eq 4 ]; then jour4+=($i); fi
  
    j=$((j+1))
  done
}

reset_stateDatas() {
  hours1=()
  hours2=()
  hours3=()
  state1=
  state2=
  state3=
  display=0
}

set_statePerHour() {
  case $valeur in
    1)
      hours1+=$heure"h "
      state1=$valeur
    ;;
    2)
      hours2+=$heure"h "
      state2=$valeur
    ;;
    *)
      hours3+=$heure"h "
      state3=$valeur
    ;;
  esac

  display=1
}

set_stateChanges() {
  for i in $2; do
    if [ "${i:0:3}" = "pas" ]; then
      heure=`echo $i|cut -d : -f2`
    else
      valeur=`echo $i|cut -d : -f2`
      [ $verbose -gt 1 ] && echo "Heure=$heure Value=$valeur default=$1"
      [ $valeur -ne $1 ] && set_statePerHour
    fi
  done
}

display_stateChanges() {
  msg="($(echo $3|tr -s - " "))"
  echo -e "\nLe $1 la situation prévue est ${status[$2]} $msg"
  echo -e "\nLe $1 la situation prévue est ${status[$2]} $msg" >> $mytomail

  if [ "x${hours1[@]}" != "x" ]; then
    echo "              à ${hours1[@]} elle sera ${status[$state1]}"
    echo "              à ${hours1[@]} elle sera ${status[$state1]}" >> $mytomail
  fi

  if [ "x${hours2[@]}" != "x" ]; then
    echo "              à ${hours2[@]} elle sera ${status[$state2]}"
    echo "              à ${hours2[@]} elle sera ${status[$state2]}" >> $mytomail
  fi

  if [ "x${hours3[@]}" != "x" ]; then
    echo "              à ${hours3[@]} elle sera ${status[$state3]}"
    echo "              à ${hours3[@]} elle sera ${status[$state3]}" >> $mytomail
  fi

  display_all=1
}

# Début de séquence
#

# Log de la date et heure d'execution du script
#
[ $logfile -eq 1 ] && echo -en ">>>>> $(date +%Y-%m-%d) " >> $mylogfile && echo -e "$(date +%0Hh:%0Mm)\n" >> $mylogfile

get_token
get_signals

# Initialisation des arrays
#
jour=()
jour1=()
jour2=()
jour3=()
jour4=()

# Chargement des dates des 4 jours
#
tmpval=`jq '.signals[]| .jour' $myoutput|cut -d T -f1|tr -d \"`
set_valuesPerDay

# Chargement de la valeur par défaut pour chacun des jours
#
tmpval=`jq '.signals[]| .dvalue' $myoutput|tr -d \"`
set_valuesPerDay

# Chargement du message du jour1 en supprimant les " . et espaces de début ou fin
# On remplace les espaces entre les mots par un tiret puis on rajoute un . final
#
tmpval=`jq '.signals[0]| .message' $myoutput`
tmpval2=$(echo $tmpval |tr -d \"|tr -d .|sed 's/^\ //'|sed 's/\ $//'|tr -s " " -)"."
jour1+=($tmpval2)

# Chargement du message du jour2, traitement identique que pour jour1
#
tmpval=`jq '.signals[1]| .message' $myoutput`
tmpval2=$(echo $tmpval |tr -d \"|tr -d .|sed 's/^\ //'|sed 's/\ $//'|tr -s " " -)"."
jour2+=($tmpval2)

# Chargement du message du jour3, traitement identique que pour jour1
#
tmpval=`jq '.signals[2]| .message' $myoutput`
tmpval2=$(echo $tmpval |tr -d \"|tr -d .|sed 's/^\ //'|sed 's/\ $//'|tr -s " " -)"."
jour3+=($tmpval2)

# Chargement du message du jour4, traitement identique que pour jour1
#
tmpval=`jq '.signals[3]| .message' $myoutput`
tmpval2=$(echo $tmpval |tr -d \"|tr -d .|sed 's/^\ //'|sed 's/\ $//'|tr -s " " -)"."
jour4+=($tmpval2)

# Pour debug
#
#echo -e ${jour1[@]}'\n'${jour2[@]}'\n'${jour3[@]}'\n'${jour4[@]}  

# Chargement des valeurs de chaque jour par tranche d'heure
# On supprime les " [ ] { les espaces et les sauts de ligne
# la séquence finale }, est remplacée par ;
#
tmpval=`jq '.signals[0]| .values[0:24]' $myoutput|tr -d \"`
tmpval=`echo $tmpval|tr -d " "|tr -d \\n|tr -d [|tr -d ]|tr -d {|tr -s }, \;`
jour1+=($tmpval)
tmpval=`jq '.signals[1]| .values[0:24]' $myoutput|tr -d \"`
tmpval=`echo $tmpval|tr -d " "|tr -d \\n|tr -d [|tr -d ]|tr -d {|tr -s }, \;`
jour2+=($tmpval)
tmpval=`jq '.signals[2]| .values[0:24]' $myoutput|tr -d \"`
tmpval=`echo $tmpval|tr -d " "|tr -d \\n|tr -d [|tr -d ]|tr -d {|tr -s }, \;`
jour3+=($tmpval)
tmpval=`jq '.signals[3]| .values[0:24]' $myoutput|tr -d \"`
tmpval=`echo $tmpval|tr -d " "|tr -d \\n|tr -d [|tr -d ]|tr -d {|tr -s }, \;`
jour4+=($tmpval)

# On extrait les dates afin de classer les dates et données par ordre croissant
#
tmpday1=$(echo ${jour1[0]}|tr -d \-)
tmpday2=$(echo ${jour2[0]}|tr -d \-)
tmpday3=$(echo ${jour3[0]}|tr -d \-)
tmpday4=$(echo ${jour4[0]}|tr -d \-)

[ $logfile -eq 1 ] && echo -e "\nClassement des dates ...: $tmpday1 $tmpday2 $tmpday3 $tmpday4" >> $mylogfile

if [ "$tmpday4" != "null." ]; then
  if [ $tmpday1 -gt $tmpday4 ]; then
    tmpday=$tmpday4
    jour=(${jour4[@]})
    tmpday4=$tmpday1
    jour4=(${jour1[@]})
    tmpday1=$tmpday
    jour1=(${jour[@]})
  fi
fi

if [ $tmpday1 -gt $tmpday3 ]; then
  tmpday=$tmpday3
  jour=(${jour3[@]})
  tmpday3=$tmpday1
  jour3=(${jour1[@]})
  tmpday1=$tmpday
  jour1=(${jour[@]})
fi

if [ $tmpday1 -gt $tmpday2 ]; then
  tmpday=$tmpday2
  jour=(${jour2[@]})
  tmpday2=$tmpday1
  jour2=(${jour1[@]})
  tmpday1=$tmpday
  jour1=(${jour[@]})
fi

if [ "$tmpday4" != "null." ]; then
  if [ $tmpday2 -gt $tmpday4 ]; then
    tmpday=$tmpday4
    jour=(${jour4[@]})
    tmpday4=$tmpday2
    jour4=(${jour2[@]})
    tmpday2=$tmpday
    jour2=(${jour[@]})
  fi
fi

if [ $tmpday2 -gt $tmpday3 ]; then
  tmpday=$tmpday3
  jour=(${jour3[@]})
  tmpday3=$tmpday2
  jour3=(${jour2[@]})
  tmpday2=$tmpday
  jour2=(${jour[@]})
fi

if [ "$tmpday4" != "null." ]; then
  if [ $tmpday3 -gt $tmpday4 ]; then
    tmpday=$tmpday4
    jour=(${jour4[@]})
    tmpday4=$tmpday3
    jour3=(${jour4[@]})
    tmpday4=$tmpday
    jour4=(${jour[@]})
  fi
fi

# Mode verbeux+1
#
[ $verbose -eq 2 ] && echo -e ${jour1[@]}'\n'${jour2[@]}'\n'${jour3[@]}'\n'${jour4[@]}  
[ $logfile -eq 1 ] && echo "Dates reclassées .......: $tmpday1 $tmpday2 $tmpday3 $tmpday4" >> $mylogfile

# le ; est notre séparateur de champs
#
IFS=';'

# Réinitialisation du fichier à envoyer par courriel (si nécessaire)
#
rm -f $mytomail
touch $mytomail

# Raz des variables de travail
#
reset_stateDatas

# Lecture des valeurs pour le 1er jour, mise en forme si alertes
#
set_stateChanges ${jour1[1]} ${jour1[3]} 
if [ $display -eq 1 -o $display_all -eq 1 ]; then
  display_stateChanges ${jour1[0]} ${jour1[1]} ${jour1[2]} 
fi
[ $logfile -eq 1 ] && echo -e "\nLe ${jour1[0]} la situation prévue est ${status[${jour1[1]}]} $msg" >> $mylogfile

# Raz des variables de travail
#
reset_stateDatas

# Lecture des valeurs pour le 2ème jour, mise en forme si alertes
#
set_stateChanges ${jour2[1]} ${jour2[3]}
if [ $display -eq 1 -o $display_all -eq 1 ]; then
  display_stateChanges ${jour2[0]} ${jour2[1]} ${jour2[2]} 
fi

[ $logfile -eq 1 ] && echo "Le ${jour2[0]} la situation prévue est ${status[${jour2[1]}]} $msg" >> $mylogfile

# Raz des variables de travail
#
reset_stateDatas

# Lecture des valeurs pour le 3ème jour, mise en forme si alertes
#
set_stateChanges ${jour3[1]} ${jour3[3]}
if [ $display -eq 1 -o $display_all -eq 1 ]; then
  display_stateChanges ${jour3[0]} ${jour3[1]} ${jour3[2]} 
fi

[ $logfile -eq 1 ] && echo "Le ${jour3[0]} la situation prévue est ${status[${jour3[1]}]} $msg" >> $mylogfile

# Raz des variables de travail
#
reset_stateDatas

# Lecture des valeurs pour le 4ème jour, mise en forme si alertes
#
if [ "${jour4[0]}" != "null." ]; then
  set_stateChanges ${jour3[1]} ${jour4[3]}
  if [ $display -eq 1 -o $display_all -eq 1 ]; then
    display_stateChanges ${jour4[0]} ${jour4[1]} ${jour4[2]} 
  fi

  [ $logfile -eq 1 ] && echo "Le ${jour4[0]} la situation prévue est ${status[${jour4[1]}]} $msg" >> $mylogfile
else
  [ $verbose -gt 0 ] && echo "Le 4ème jour vaut ${jour4[0]} : aucune donnée, une nouvelle période de 4 jours débute"
  [ $logfile -eq 1 ] && echo "Le 4ème jour vaut ${jour4[0]} : aucune donnée, une nouvelle période de 4 jours débute" >> $mylogfile
fi

# IFS est remis à sa valeur standard
#
IFS=$BAKIFS

# Un courriel n'est envoyé que s'il contient des valeurs (taille supérieure à zéro)
#
tomailsize=$(wc -c $mytomail|cut -d " " -f1)
[ $tomailsize -gt 0 ] && mail -s "Information Ecowatt" $mailalert < $mytomail 

if [ $verbose -gt 0 ]; then
  [ $tomailsize -gt 0 ] && echo -e "\nUn courriel est envoyé\n" || echo -e "\nPas courriel généré, tout est au vert\n"
fi

if [ $logfile -eq 1 ]; then
  if [ $tomailsize -gt 0 ]; then
    echo -e "\nUn courriel est envoyé" >> $mylogfile
  else
    echo -e "\nPas de courriel généré, tout est au vert" >> $mylogfile
  fi
fi

[ $verbose -gt 0 ] && echo -e "\nFin de traitement" || echo ""
[ $logfile -eq 1 ] && echo -e "\n<<<<< Fin de traitement" >> $mylogfile

exit 0
